@@ -22,7 +22,7 @@ Bifrost Inventory Module
 This is a dynamic inventory module intended to provide a platform for
 consistent inventory information for Bifrost.
 
-The inventory currently supplies two distinct groups:
+The inventory supplies two distinct groups by default:
 
     - localhost
     - baremetal
@@ -35,6 +35,21 @@ variables extracted from the data source. The variables are defined on a
 per-host level which allows explicit actions to be taken based upon the
 variables.
 
+It is also possible for users to specify additional per-host groups by
+simply setting the host_groups variable in the inventory file. See below for
+an example JSON file.
+
+The default group can also be changed by setting the DEFAULT_HOST_GROUPS
+variable to contain the desired groups separated by whitespace as follows:
+
+DEFAULT_HOST_GROUPS="foo bar zoo"
+
+In case of provisioning virtual machines, additional per-VM groups can
+be set by simply setting the test_vm_groups[$host] variable to a list
+of desired groups. Moreover, users can override the default 'baremetal'
+group by assigning a list of default groups to the test_vm_default_group
+variable.
+
 Presently, the base mode of operation reads a CSV file in the format
 originally utilized by bifrost and returns structured JSON that is
 interpreted by Ansible.  This has since been extended to support the
@@ -62,6 +77,10 @@ Example JSON Element:
 {
   "node1": {
     "uuid": "a8cb6624-0d9f-c882-affc-046ebb96ec01",
+    "host_groups": [
+        "nova",
+        "neutron"
+    ],
     "driver_info": {
       "power": {
         "ipmi_target_channel": "0",
@@ -80,7 +99,7 @@ Example JSON Element:
         "mac": "00:01:02:03:04:06"
       }
    ],
-   "driver": "agent_ipmitool",
+   "driver": "ipmi",
    "ipv4_address": "192.168.122.2",
    "properties": {
       "cpu_arch": "x86_64",
@@ -163,8 +182,8 @@ def _val_or_none(array, location):
             return array[location]
         return None
     except IndexError:
-        LOG.debug(("Out of range value encountered.  Requested "
-                  "field %s Had: %s" % (location, array)))
+        LOG.debug("Out of range value encountered.  Requested "
+                  "field %s Had: %s", location, array)
 
 
 def _process_baremetal_data(data_source, groups, hostvars):
@@ -173,7 +192,7 @@ def _process_baremetal_data(data_source, groups, hostvars):
         try:
             file_data = yaml.safe_load(file_object)
         except Exception as e:
-            LOG.debug("Failed to parse JSON or YAML: %s" % e)
+            LOG.debug("Failed to parse JSON or YAML: %s", e)
             raise Exception("Failed to parse JSON or YAML")
 
     for name in file_data:
@@ -181,6 +200,11 @@ def _process_baremetal_data(data_source, groups, hostvars):
         # Perform basic validation
         node_net_data = host.get('node_network_data')
         ipv4_addr = host.get('ipv4_address')
+        default_groups = os.environ.get('DEFAULT_HOST_GROUPS',
+                                        'baremetal').split()
+        host['host_groups'] = sorted(list(set(host.get('host_groups', []) +
+                                              default_groups)))
+
         if not node_net_data and not ipv4_addr:
             host['addressing_mode'] = "dhcp"
         else:
@@ -190,7 +214,10 @@ def _process_baremetal_data(data_source, groups, hostvars):
                 'addressing_mode' not in host):
             host['provisioning_ipv4_address'] = host['ipv4_address']
         # Add each host to the values to be returned.
-        groups['baremetal']['hosts'].append(host['name'])
+        for group in host['host_groups']:
+            if group not in groups:
+                groups.update({group: {'hosts': []}})
+            groups[group]['hosts'].append(host['name'])
         hostvars.update({host['name']: host})
     return (groups, hostvars)
 
@@ -226,6 +253,7 @@ def _process_baremetal_csv(data_source, groups, hostvars):
             properties['cpu_arch'] = "x86_64"
             host['uuid'] = _val_or_none(row, 9)
             host['name'] = _val_or_none(row, 10)
+            host['host_groups'] = ["baremetal"]
             host['ipv4_address'] = _val_or_none(row, 11)
             if ('ipv4_address' not in host or
                     not host['ipv4_address']):
@@ -241,7 +269,7 @@ def _process_baremetal_csv(data_source, groups, hostvars):
                 host['provisioning_ipv4_address'] = host['ipv4_address']
 
             # Default Driver unless otherwise defined or determined.
-            host['driver'] = "agent_ssh"
+            host['driver'] = "ipmi"
 
             if len(row) > 15:
                 driver = _val_or_none(row, 16)
@@ -249,8 +277,8 @@ def _process_baremetal_csv(data_source, groups, hostvars):
                     host['driver'] = driver
 
             if "ipmi" in host['driver']:
-                # Set agent_ipmitool by default
-                host['driver'] = "agent_ipmitool"
+                # Set ipmi by default
+                host['driver'] = "ipmi"
                 power['ipmi_address'] = management_address
                 power['ipmi_username'] = management_username
                 power['ipmi_password'] = management_password
@@ -267,22 +295,6 @@ def _process_baremetal_csv(data_source, groups, hostvars):
                             power['ipmi_transit_address']):
                         power['ipmi_bridging'] = 'dual'
 
-            if "ssh" in host['driver']:
-                # Under another model, a user would define
-                # and value translations to load these
-                # values.  Since we're supporting the base
-                # model bifrost was developed with, then
-                # we need to make sure these are present as
-                # they are expected values.
-                power['ssh_virt_type'] = "virsh"
-                power['ssh_address'] = management_address
-                power['ssh_port'] = 22
-                # NOTE: The CSV format is desynced from the enrollment
-                # playbook at present, so we're hard coding ironic here
-                # as that is what the test is known to work with.
-                power['ssh_username'] = "ironic"
-                power['ssh_key_filename'] = "/home/ironic/.ssh/id_rsa"
-
             # Group variables together under host.
             # NOTE(TheJulia): Given the split that this demonstrates, where
             # deploy details could possible be imported from a future
@@ -336,8 +348,8 @@ def _process_shade(groups, hostvars):
         # return the network information to the inventory.
         nics = cloud.list_nics_for_machine(machine['uuid'])
         new_nics = []
-        new_nic = {}
         for nic in nics:
+            new_nic = {}
             if 'address' in nic:
                 new_nic['mac'] = nic['address']
             new_nics.append(new_nic)
@@ -373,7 +385,7 @@ def main():
                     groups,
                     hostvars)
             except Exception as e:
-                LOG.debug("File does not appear to be JSON or YAML - %s" % e)
+                LOG.debug("File does not appear to be JSON or YAML - %s", e)
                 try:
                     (groups, hostvars) = _process_baremetal_csv(
                         data_source,
@@ -381,7 +393,7 @@ def main():
                         hostvars)
                 except Exception as e:
                     LOG.debug("CSV fallback processing failed, "
-                              "received: &s" % e)
+                              "received: %s", e)
                     LOG.error("BIFROST_INVENTORY_SOURCE does not define "
                               "a file that could be processed: "
                               "Tried JSON, YAML, and CSV formats")
@@ -402,6 +414,14 @@ def main():
         LOG.error('Failed processing: %s' % error)
         sys.exit(1)
 
+    # Drop empty groups. This is usually necessary when
+    # the default ["baremetal"] group has been overridden
+    # by the user.
+    for group in groups.keys():
+        # Empty groups
+        if len(groups[group]['hosts']) == 0:
+            del groups[group]
+
     # General Data Conversion
 
     if not config.convertcsv:
