@@ -3,6 +3,9 @@ set -eu
 
 declare -A PKG_MAP
 
+# workaround: for latest bindep to work, it needs to use en_US local
+export LANG=c
+
 CHECK_CMD_PKGS=(
     gcc
     libffi
@@ -16,9 +19,9 @@ CHECK_CMD_PKGS=(
     wget
 )
 
-# Check zypper before apt-get in case zypper-aptitude
-# is installed
-if [ -x '/usr/bin/zypper' ]; then
+source /etc/os-release || source /usr/lib/os-release
+case ${ID,,} in
+    *suse)
     OS_FAMILY="Suse"
     INSTALLER_CMD="sudo -H -E zypper install -y --no-recommends"
     CHECK_CMD="zypper search --match-exact --installed"
@@ -35,13 +38,17 @@ if [ -x '/usr/bin/zypper' ]; then
         [wget]=wget
     )
     EXTRA_PKG_DEPS=( python-xml )
+    sudo zypper -n ref
     # NOTE (cinerama): we can't install python without removing this package
     # if it exists
     if $(${CHECK_CMD} patterns-openSUSE-minimal_base-conflicts &> /dev/null); then
         sudo -H zypper remove -y patterns-openSUSE-minimal_base-conflicts
     fi
-elif [ -x '/usr/bin/apt-get' ]; then
+    ;;
+
+    ubuntu|debian)
     OS_FAMILY="Debian"
+    export DEBIAN_FRONTEND=noninteractive
     INSTALLER_CMD="sudo -H -E apt-get -y install"
     CHECK_CMD="dpkg -l"
     PKG_MAP=(
@@ -57,7 +64,10 @@ elif [ -x '/usr/bin/apt-get' ]; then
         [wget]=wget
     )
     EXTRA_PKG_DEPS=()
-elif [ -x '/usr/bin/dnf' ] || [ -x '/usr/bin/yum' ]; then
+    sudo apt-get update
+    ;;
+
+    rhel|fedora|centos)
     OS_FAMILY="RedHat"
     PKG_MANAGER=$(which dnf || which yum)
     INSTALLER_CMD="sudo -H -E ${PKG_MANAGER} -y install"
@@ -75,9 +85,14 @@ elif [ -x '/usr/bin/dnf' ] || [ -x '/usr/bin/yum' ]; then
         [wget]=wget
     )
     EXTRA_PKG_DEPS=()
-else
-    echo "ERROR: Supported package manager not found.  Supported: apt, dnf, yum, zypper"
-fi
+    sudo yum updateinfo
+    if $(grep -q Fedora /etc/redhat-release); then
+        EXTRA_PKG_DEPS="python-dnf redhat-rpm-config"
+    fi
+    ;;
+
+    *) echo "ERROR: Supported package manager not found.  Supported: apt, dnf, yum, zypper"; exit 1;;
+esac
 
 # if running in OpenStack CI, then make sure epel is enabled
 # since it may already be present (but disabled) on the host
@@ -126,7 +141,7 @@ if [ -n "${VENV-}" ]; then
     # Note(cinerama): activate is not compatible with "set -u";
     # disable it just for this line.
     set +u
-    source ${VENV}/bin/activate
+    . ${VENV}/bin/activate
     set -u
     VIRTUAL_ENV=${VENV}
 else
@@ -158,7 +173,7 @@ if [[ $(readlink -f /etc/alternatives/pip) =~ "pip3" ]]; then
 fi
 
 if ! which pip; then
-    wget -O /tmp/get-pip.py https://bootstrap.pypa.io/3.2/get-pip.py
+    wget -O /tmp/get-pip.py https://bootstrap.pypa.io/get-pip.py
     sudo -H -E ${PYTHON} /tmp/get-pip.py
 fi
 
@@ -172,3 +187,6 @@ sudo -H -E ${PIP} install bindep
 
 # bindep returns 1 if packages are missing
 bindep -b &> /dev/null || ${INSTALLER_CMD} $(bindep -b)
+
+# upgrade setuptools, as latest version is needed to install some projects
+sudo -H -E ${PIP} install --upgrade setuptools
